@@ -7,8 +7,7 @@ button = Button(2)
 camera = PiCamera()
 
 def capture():
-    timestamp = datetime.now().isoformat()
-    camera.capture('/home/pi/%s.jpg' % timestamp)
+    camera.capture(f'/home/pi/{datetime.now():%Y-%m-%d-%H-%M-%S}.jpg')
 
 button.when_pressed = capture
 
