@@ -1,9 +1,10 @@
-from gpiozero import Button, Robot
+from gpiozero import Button, Robot, Motor
 from gpiozero.pins.pigpio import PiGPIOFactory
 from signal import pause
 
 factory = PiGPIOFactory(host='192.168.1.17')
-robot = Robot(left=(4, 14), right=(17, 18), pin_factory=factory)  # remote pins
+robot = Robot(left=Motor(4, 14), right=Motor(17, 18),
+              pin_factory=factory)  # remote pins
 
 # local buttons
 left = Button(26)
