@@ -1,8 +1,8 @@
-from gpiozero import Robot, DistanceSensor
+from gpiozero import Robot, Motor, DistanceSensor
 from signal import pause
 
 sensor = DistanceSensor(23, 24, max_distance=1, threshold_distance=0.2)
-robot = Robot(left=(4, 14), right=(17, 18))
+robot = Robot(left=Motor(4, 14), right=Motor(17, 18))
 
 sensor.when_in_range = robot.backward
 sensor.when_out_of_range = robot.stop
