@@ -1,8 +1,8 @@
-from gpiozero import Robot, MCP3008
+from gpiozero import Robot, Motor, MCP3008
 from gpiozero.tools import zip_values
 from signal import pause
 
-robot = Robot(left=(4, 14), right=(17, 18))
+robot = Robot(left=Motor(4, 14), right=Motor(17, 18))
 
 left_pot = MCP3008(0)
 right_pot = MCP3008(1)
