@@ -2,9 +2,10 @@
 #
 # GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
 #
-# Copyright (c) 2015-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2015-2023 Dave Jones <dave@waveform.org.uk>
 # Copyright (c) 2015-2021 Ben Nuttall <ben@bennuttall.com>
 # Copyright (c) 2020 Robert Erdin <roberte@depop.com>
+# Copyright (c) 2020 Fangchen Li <fangchen.li@outlook.com>
 # Copyright (c) 2020 Dan Jackson <dan@djackson.org>
 # Copyright (c) 2016-2020 Andrew Scheller <github@loowis.durge.org>
 # Copyright (c) 2019 Kosovan Sofiia <sofiia.kosovan@gmail.com>
@@ -13,24 +14,19 @@
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
-from __future__ import (
-    unicode_literals,
-    print_function,
-    absolute_import,
-    division,
-)
-
 import warnings
-from time import sleep, time
+from time import sleep
 from threading import Event, Lock
 from itertools import tee
-try:
-    from statistics import median, mean
-except ImportError:
-    from .compat import median, mean
+from statistics import median, mean
 
-from .exc import InputDeviceError, DeviceClosed, DistanceSensorNoEcho, \
-    PinInvalidState, PWMSoftwareFallback
+from .exc import (
+    InputDeviceError,
+    DeviceClosed,
+    DistanceSensorNoEcho,
+    PinInvalidState,
+    PWMSoftwareFallback,
+)
 from .devices import GPIODevice, CompositeDevice
 from .mixins import GPIOQueue, EventsMixin, HoldMixin, event
 try:
@@ -78,9 +74,9 @@ class InputDevice(GPIODevice):
         See :doc:`api_pins` for more information (this is an advanced feature
         which most users can ignore).
     """
-    def __init__(self, pin=None, pull_up=False, active_state=None,
+    def __init__(self, pin=None, *, pull_up=False, active_state=None,
                  pin_factory=None):
-        super(InputDevice, self).__init__(pin, pin_factory=pin_factory)
+        super().__init__(pin, pin_factory=pin_factory)
         try:
             self.pin.function = 'input'
             pull = {None: 'floating', True: 'up', False: 'down'}[pull_up]
@@ -93,14 +89,14 @@ class InputDevice(GPIODevice):
         if pull_up is None:
             if active_state is None:
                 raise PinInvalidState(
-                    'Pin %d is defined as floating, but "active_state" is not '
-                    'defined' % self.pin.number)
+                    f'Pin {self.pin.info.name} is defined as floating, but '
+                    f'"active_state" is not defined')
             self._active_state = bool(active_state)
         else:
             if active_state is not None:
                 raise PinInvalidState(
-                    'Pin %d is not floating, but "active_state" is not None' %
-                    self.pin.number)
+                    f'Pin {self.pin.info.name} is not floating, but '
+                    f'"active_state" is not None')
             self._active_state = False if pull_up else True
         self._inactive_state = not self._active_state
 
@@ -118,10 +114,12 @@ class InputDevice(GPIODevice):
 
     def __repr__(self):
         try:
-            return "<gpiozero.%s object on pin %r, pull_up=%s, is_active=%s>" % (
-                self.__class__.__name__, self.pin, self.pull_up, self.is_active)
+            return (
+                f"<gpiozero.{self.__class__.__name__} object on pin "
+                f"{self.pin!r}, pull_up={self.pull_up}, "
+                f"is_active={self.is_active}>")
         except:
-            return super(InputDevice, self).__repr__()
+            return super().__repr__()
 
 
 class DigitalInputDevice(EventsMixin, InputDevice):
@@ -159,10 +157,9 @@ class DigitalInputDevice(EventsMixin, InputDevice):
         See :doc:`api_pins` for more information (this is an advanced feature
         which most users can ignore).
     """
-    def __init__(
-            self, pin=None, pull_up=False, active_state=None, bounce_time=None,
-            pin_factory=None):
-        super(DigitalInputDevice, self).__init__(
+    def __init__(self, pin=None, *, pull_up=False, active_state=None,
+                 bounce_time=None, pin_factory=None):
+        super().__init__(
             pin, pull_up=pull_up, active_state=active_state,
             pin_factory=pin_factory)
         try:
@@ -253,11 +250,11 @@ class SmoothedInputDevice(EventsMixin, InputDevice):
         which most users can ignore).
     """
     def __init__(
-            self, pin=None, pull_up=False, active_state=None, threshold=0.5,
+            self, pin=None, *, pull_up=False, active_state=None, threshold=0.5,
             queue_len=5, sample_wait=0.0, partial=False, average=median,
             ignore=None, pin_factory=None):
         self._queue = None
-        super(SmoothedInputDevice, self).__init__(
+        super().__init__(
             pin, pull_up=pull_up, active_state=active_state,
             pin_factory=pin_factory)
         try:
@@ -280,19 +277,20 @@ class SmoothedInputDevice(EventsMixin, InputDevice):
             # because we're trying to close the thread anyway
             pass
         self._queue = None
-        super(SmoothedInputDevice, self).close()
+        super().close()
 
     def __repr__(self):
         try:
             self._check_open()
         except DeviceClosed:
-            return super(SmoothedInputDevice, self).__repr__()
+            return super().__repr__()
         else:
             if self.partial or self._queue.full.is_set():
-                return super(SmoothedInputDevice, self).__repr__()
+                return super().__repr__()
             else:
-                return "<gpiozero.%s object on pin %r, pull_up=%s>" % (
-                    self.__class__.__name__, self.pin, self.pull_up)
+                return (
+                    f"<gpiozero.{self.__class__.__name__} object on pin "
+                    f"{self.pin!r}, pull_up={self.pull_up}>")
 
     @property
     def queue_len(self):
@@ -408,10 +406,10 @@ class Button(HoldMixin, DigitalInputDevice):
         See :doc:`api_pins` for more information (this is an advanced feature
         which most users can ignore).
     """
-    def __init__(
-            self, pin=None, pull_up=True, active_state=None, bounce_time=None,
-            hold_time=1, hold_repeat=False, pin_factory=None):
-        super(Button, self).__init__(
+    def __init__(self, pin=None, *, pull_up=True, active_state=None,
+                 bounce_time=None, hold_time=1, hold_repeat=False,
+                 pin_factory=None):
+        super().__init__(
             pin, pull_up=pull_up, active_state=active_state,
             bounce_time=bounce_time, pin_factory=pin_factory)
         self.hold_time = hold_time
@@ -422,7 +420,7 @@ class Button(HoldMixin, DigitalInputDevice):
         """
         Returns 1 if the button is currently pressed, and 0 if it is not.
         """
-        return super(Button, self).value
+        return super().value
 
 Button.is_pressed = Button.is_active
 Button.pressed_time = Button.active_time
@@ -495,10 +493,10 @@ class LineSensor(SmoothedInputDevice):
 
     .. _CamJam #3 EduKit: http://camjam.me/?page_id=1035
     """
-    def __init__(
-            self, pin=None, pull_up=False, active_state=None, queue_len=5,
-            sample_rate=100, threshold=0.5, partial=False, pin_factory=None):
-        super(LineSensor, self).__init__(
+    def __init__(self, pin=None, *, pull_up=False, active_state=None,
+                 queue_len=5, sample_rate=100, threshold=0.5, partial=False,
+                 pin_factory=None):
+        super().__init__(
             pin, pull_up=pull_up, active_state=active_state,
             threshold=threshold, queue_len=queue_len,
             sample_wait=1 / sample_rate, partial=partial,
@@ -512,7 +510,7 @@ class LineSensor(SmoothedInputDevice):
         is nearer 0 for black under the sensor, and nearer 1 for white under
         the sensor.
         """
-        return super(LineSensor, self).value
+        return super().value
 
     @property
     def line_detected(self):
@@ -584,10 +582,10 @@ class MotionSensor(SmoothedInputDevice):
         See :doc:`api_pins` for more information (this is an advanced feature
         which most users can ignore).
     """
-    def __init__(
-            self, pin=None, pull_up=False, active_state=None, queue_len=1,
-            sample_rate=10, threshold=0.5, partial=False, pin_factory=None):
-        super(MotionSensor, self).__init__(
+    def __init__(self, pin=None, *, pull_up=False, active_state=None,
+                 queue_len=1, sample_rate=10, threshold=0.5, partial=False,
+                 pin_factory=None):
+        super().__init__(
             pin, pull_up=pull_up, active_state=active_state,
             threshold=threshold, queue_len=queue_len, sample_wait=1 /
             sample_rate, partial=partial, pin_factory=pin_factory, average=mean)
@@ -601,7 +599,7 @@ class MotionSensor(SmoothedInputDevice):
         a *queue_len* greater than 1, this will be an averaged value where
         values closer to 1 imply motion detection.
         """
-        return super(MotionSensor, self).value
+        return super().value
 
 MotionSensor.motion_detected = MotionSensor.is_active
 MotionSensor.when_motion = MotionSensor.when_activated
@@ -664,10 +662,9 @@ class LightSensor(SmoothedInputDevice):
 
     .. _CamJam #2 EduKit: http://camjam.me/?page_id=623
     """
-    def __init__(
-            self, pin=None, queue_len=5, charge_time_limit=0.01,
-            threshold=0.1, partial=False, pin_factory=None):
-        super(LightSensor, self).__init__(
+    def __init__(self, pin=None, *, queue_len=5, charge_time_limit=0.01,
+                 threshold=0.1, partial=False, pin_factory=None):
+        super().__init__(
             pin, pull_up=False, threshold=threshold, queue_len=queue_len,
             sample_wait=0.0, partial=partial, pin_factory=pin_factory)
         try:
@@ -713,7 +710,7 @@ class LightSensor(SmoothedInputDevice):
         """
         Returns a value between 0 (dark) and 1 (light).
         """
-        return super(LightSensor, self).value
+        return super().value
 
 LightSensor.light_detected = LightSensor.is_active
 LightSensor.when_light = LightSensor.when_activated
@@ -823,11 +820,11 @@ class DistanceSensor(SmoothedInputDevice):
     """
     ECHO_LOCK = Lock()
 
-    def __init__(
-            self, echo=None, trigger=None, queue_len=9, max_distance=1,
-            threshold_distance=0.3, partial=False, pin_factory=None):
+    def __init__(self, echo=None, trigger=None, *, queue_len=9,
+                 max_distance=1, threshold_distance=0.3, partial=False,
+                 pin_factory=None):
         self._trigger = None
-        super(DistanceSensor, self).__init__(
+        super().__init__(
             echo, pull_up=False, queue_len=queue_len, sample_wait=0.06,
             partial=partial, ignore=frozenset({None}), pin_factory=pin_factory
         )
@@ -863,7 +860,7 @@ class DistanceSensor(SmoothedInputDevice):
         except AttributeError:
             pass
         self._trigger = None
-        super(DistanceSensor, self).close()
+        super().close()
 
     @property
     def max_distance(self):
@@ -915,7 +912,7 @@ class DistanceSensor(SmoothedInputDevice):
         difference, and 1, indicating the reflector is at or beyond the
         specified *max_distance*.
         """
-        return super(DistanceSensor, self).value
+        return super().value
 
     @property
     def trigger(self):
@@ -1129,9 +1126,8 @@ class RotaryEncoder(EventsMixin, CompositeDevice):
     # practice with several encoders, even quite jiggly ones with no debounce
     # hardware or software
 
-    def __init__(
-            self, a, b, bounce_time=None, max_steps=16, threshold_steps=(0, 0),
-            wrap=False, pin_factory=None):
+    def __init__(self, a, b, *, bounce_time=None, max_steps=16,
+                 threshold_steps=(0, 0), wrap=False, pin_factory=None):
         min_thresh, max_thresh = threshold_steps
         if max_thresh < min_thresh:
             raise ValueError('maximum threshold cannot be less than minimum')
@@ -1147,7 +1143,7 @@ class RotaryEncoder(EventsMixin, CompositeDevice):
         self._rotate_event = Event()
         self._rotate_cw_event = Event()
         self._rotate_ccw_event = Event()
-        super(RotaryEncoder, self).__init__(
+        super().__init__(
             a=InputDevice(a, pull_up=True, pin_factory=pin_factory),
             b=InputDevice(b, pull_up=True, pin_factory=pin_factory),
             _order=('a', 'b'), pin_factory=pin_factory)
@@ -1163,10 +1159,11 @@ class RotaryEncoder(EventsMixin, CompositeDevice):
     def __repr__(self):
         try:
             self._check_open()
-            return "<gpiozero.%s object on pins %r and %r>" % (
-                self.__class__.__name__, self.a.pin, self.b.pin)
+            return (
+                f"<gpiozero.{self.__class__.__name__} object on pins "
+                f"{self.a.pin!r} and {self.b.pin!r}>")
         except DeviceClosed:
-            return super(RotaryEncoder, self).__repr__()
+            return super().__repr__()
 
     def _a_changed(self, ticks, state):
         edge = (self.a._state_to_value(state) << 1) | (self._edge & 0x1)
