@@ -2,27 +2,18 @@
 #
 # GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
 #
-# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2016-2023 Dave Jones <dave@waveform.org.uk>
 # Copyright (c) 2017-2021 Ben Nuttall <ben@bennuttall.com>
 # Copyright (c) 2019 Jeevan M R <14.jeevan@gmail.com>
 # Copyright (c) 2019 Andrew Scheller <github@loowis.durge.org>
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
-from __future__ import (
-    unicode_literals,
-    print_function,
-    absolute_import,
-    division,
-)
-str = type('')
-
-
 import os
 import io
+import warnings
 import subprocess
 from datetime import datetime, time
-import warnings
 
 from .devices import Device
 from .mixins import EventsMixin, event
@@ -37,13 +28,13 @@ class InternalDevice(EventsMixin, Device):
     usually represent operating system services like the internal clock, file
     systems or network facilities.
     """
-    def __init__(self, pin_factory=None):
+    def __init__(self, *, pin_factory=None):
         self._closed = False
-        super(InternalDevice, self).__init__(pin_factory=pin_factory)
+        super().__init__(pin_factory=pin_factory)
 
     def close(self):
         self._closed = True
-        super(InternalDevice, self).close()
+        super().close()
 
     @property
     def closed(self):
@@ -52,9 +43,9 @@ class InternalDevice(EventsMixin, Device):
     def __repr__(self):
         try:
             self._check_open()
-            return "<gpiozero.%s object>" % self.__class__.__name__
+            return f"<gpiozero.{self.__class__.__name__} object>"
         except DeviceClosed:
-            return "<gpiozero.%s object closed>" % self.__class__.__name__
+            return f"<gpiozero.{self.__class__.__name__} object closed>"
 
 
 class PolledInternalDevice(InternalDevice):
@@ -63,17 +54,17 @@ class PolledInternalDevice(InternalDevice):
     internal devices that lack any other mechanism to inform the instance of
     changes.
     """
-    def __init__(self, event_delay=1.0, pin_factory=None):
+    def __init__(self, *, event_delay=1.0, pin_factory=None):
         self._event_thread = None
         self._event_delay = event_delay
-        super(PolledInternalDevice, self).__init__(pin_factory=pin_factory)
+        super().__init__(pin_factory=pin_factory)
 
     def close(self):
         try:
             self._start_stop_events(False)
         except AttributeError:
             pass  # pragma: no cover
-        super(PolledInternalDevice, self).close()
+        super().close()
 
     @property
     def event_delay(self):
@@ -97,7 +88,7 @@ class PolledInternalDevice(InternalDevice):
     def wait_for_active(self, timeout=None):
         self._start_stop_events(True)
         try:
-            return super(PolledInternalDevice, self).wait_for_active(timeout)
+            return super().wait_for_active(timeout)
         finally:
             self._start_stop_events(
                 self.when_activated or self.when_deactivated)
@@ -105,7 +96,7 @@ class PolledInternalDevice(InternalDevice):
     def wait_for_inactive(self, timeout=None):
         self._start_stop_events(True)
         try:
-            return super(PolledInternalDevice, self).wait_for_inactive(timeout)
+            return super().wait_for_inactive(timeout)
         finally:
             self._start_stop_events(
                 self.when_activated or self.when_deactivated)
@@ -153,18 +144,17 @@ class PingServer(PolledInternalDevice):
         See :doc:`api_pins` for more information (this is an advanced feature
         which most users can ignore).
     """
-    def __init__(self, host, event_delay=10.0, pin_factory=None):
+    def __init__(self, host, *, event_delay=10.0, pin_factory=None):
         self._host = host
-        super(PingServer, self).__init__(
-            event_delay=event_delay, pin_factory=pin_factory)
+        super().__init__(event_delay=event_delay, pin_factory=pin_factory)
         self._fire_events(self.pin_factory.ticks(), self.is_active)
 
     def __repr__(self):
         try:
             self._check_open()
-            return '<gpiozero.PingServer object host="%s">' % self.host
+            return f'<gpiozero.PingServer object host="{self.host}">'
         except DeviceClosed:
-            return super(PingServer, self).__repr__()
+            return super().__repr__()
 
     @property
     def host(self):
@@ -236,7 +226,7 @@ class CPUTemperature(PolledInternalDevice):
         # bar graph is a bit more "lively"
         cpu = CPUTemperature(min_temp=50, max_temp=90)
 
-        print('Initial temperature: {}C'.format(cpu.temperature))
+        print(f'Initial temperature: {cpu.temperature}C')
 
         graph = LEDBarGraph(5, 6, 13, 19, 25, pwm=True)
         graph.source = cpu
@@ -270,12 +260,11 @@ class CPUTemperature(PolledInternalDevice):
         See :doc:`api_pins` for more information (this is an advanced feature
         which most users can ignore).
     """
-    def __init__(self, sensor_file='/sys/class/thermal/thermal_zone0/temp',
+    def __init__(self, sensor_file='/sys/class/thermal/thermal_zone0/temp', *,
             min_temp=0.0, max_temp=100.0, threshold=80.0, event_delay=5.0,
             pin_factory=None):
         self.sensor_file = sensor_file
-        super(CPUTemperature, self).__init__(
-            event_delay=event_delay, pin_factory=pin_factory)
+        super().__init__(event_delay=event_delay, pin_factory=pin_factory)
         try:
             if min_temp >= max_temp:
                 raise ValueError('max_temp must be greater than min_temp')
@@ -293,9 +282,11 @@ class CPUTemperature(PolledInternalDevice):
     def __repr__(self):
         try:
             self._check_open()
-            return '<gpiozero.CPUTemperature object temperature=%.2f>' % self.temperature
+            return (
+                f'<gpiozero.{self.__class__.__name__} object '
+                f'temperature={self.temperature:.2f}>')
         except DeviceClosed:
-            return super(CPUTemperature, self).__repr__()
+            return super().__repr__()
 
     @property
     def temperature(self):
@@ -402,9 +393,9 @@ class LoadAverage(PolledInternalDevice):
         See :doc:`api_pins` for more information (this is an advanced feature
         which most users can ignore).
     """
-    def __init__(self, load_average_file='/proc/loadavg', min_load_average=0.0,
-        max_load_average=1.0, threshold=0.8, minutes=5, event_delay=10.0,
-        pin_factory=None):
+    def __init__(self, load_average_file='/proc/loadavg', *,
+                 min_load_average=0.0, max_load_average=1.0, threshold=0.8,
+                 minutes=5, event_delay=10.0, pin_factory=None):
         if min_load_average >= max_load_average:
             raise ValueError(
                 'max_load_average must be greater than min_load_average')
@@ -423,16 +414,17 @@ class LoadAverage(PolledInternalDevice):
             5: 1,
             15: 2,
         }[minutes]
-        super(LoadAverage, self).__init__(
-            event_delay=event_delay, pin_factory=pin_factory)
+        super().__init__(event_delay=event_delay, pin_factory=pin_factory)
         self._fire_events(self.pin_factory.ticks(), None)
 
     def __repr__(self):
         try:
             self._check_open()
-            return '<gpiozero.LoadAverage object load average=%.2f>' % self.load_average
+            return (
+                f'<gpiozero.{self.__class__.__name__} object '
+                f'load average={self.load_average:.2f}>')
         except DeviceClosed:
-            return super(LoadAverage, self).__repr__()
+            return super().__repr__()
 
     @property
     def load_average(self):
@@ -440,7 +432,6 @@ class LoadAverage(PolledInternalDevice):
         Returns the current load average.
         """
         with io.open(self.load_average_file, 'r') as f:
-            print(repr(f))
             file_columns = f.read().strip().split()
             return float(file_columns[self._load_average_file_column])
 
@@ -535,13 +526,12 @@ class TimeOfDay(PolledInternalDevice):
         See :doc:`api_pins` for more information (this is an advanced feature
         which most users can ignore).
     """
-    def __init__(self, start_time, end_time, utc=True, event_delay=5.0,
+    def __init__(self, start_time, end_time, *, utc=True, event_delay=5.0,
                  pin_factory=None):
         self._start_time = None
         self._end_time = None
         self._utc = True
-        super(TimeOfDay, self).__init__(
-            event_delay=event_delay, pin_factory=pin_factory)
+        super().__init__(event_delay=event_delay, pin_factory=pin_factory)
         try:
             self._start_time = self._validate_time(start_time)
             self._end_time = self._validate_time(end_time)
@@ -556,10 +546,12 @@ class TimeOfDay(PolledInternalDevice):
     def __repr__(self):
         try:
             self._check_open()
-            return '<gpiozero.TimeOfDay object active between %s and %s %s>' % (
-                self.start_time, self.end_time, ('local', 'UTC')[self.utc])
+            return (
+                f'<gpiozero.{self.__class__.__name__} object active between '
+                f'{self.start_time} and {self.end_time} '
+                f'{("local", "UTC")[self.utc]}>')
         except DeviceClosed:
-            return super(TimeOfDay, self).__repr__()
+            return super().__repr__()
 
     def _validate_time(self, value):
         if isinstance(value, datetime):
@@ -647,7 +639,7 @@ class DiskUsage(PolledInternalDevice):
 
         disk = DiskUsage()
 
-        print('Current disk usage: {}%'.format(disk.usage))
+        print(f'Current disk usage: {disk.usage}%')
 
         graph = LEDBarGraph(5, 6, 13, 19, 25, pwm=True)
         graph.source = disk
@@ -671,9 +663,9 @@ class DiskUsage(PolledInternalDevice):
         See :doc:`api_pins` for more information (this is an advanced feature
         which most users can ignore).
     """
-    def __init__(self, filesystem='/', threshold=90.0, event_delay=30.0,
+    def __init__(self, filesystem='/', *, threshold=90.0, event_delay=30.0,
                  pin_factory=None):
-        super(DiskUsage, self).__init__(
+        super().__init__(
             event_delay=event_delay, pin_factory=pin_factory)
         os.statvfs(filesystem)
         if not 0 <= threshold <= 100:
@@ -686,9 +678,11 @@ class DiskUsage(PolledInternalDevice):
     def __repr__(self):
         try:
             self._check_open()
-            return '<gpiozero.DiskUsage object usage=%.2f>' % self.usage
+            return (
+                f'<gpiozero.{self.__class__.__name__} object '
+                f'usage={self.usage:.2f}>')
         except DeviceClosed:
-            return super(DiskUsage, self).__repr__()
+            return super().__repr__()
 
     @property
     def usage(self):
