@@ -1,22 +1,21 @@
-from __future__ import (
-    unicode_literals,
-    print_function,
-    absolute_import,
-    division,
-    )
-nstr = str
-str = type('')
+# vim: set fileencoding=utf-8:
+#
+# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
+#
+# Copyright (c) 2016-2023 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2018-2021 Ben Nuttall <ben@bennuttall.com>
+# Copyright (c) 2020 Fangchen Li <fangchen.li@outlook.com>
+# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
+#
+# SPDX-License-Identifier: BSD-3-Clause
 
 import inspect
 import weakref
+import warnings
 from functools import wraps, partial
 from threading import Event
 from collections import deque
-from time import time
-try:
-    from statistics import median
-except ImportError:
-    from .compat import median
+from statistics import median
 
 from .threads import GPIOThread
 from .exc import (
@@ -24,14 +23,21 @@ from .exc import (
     BadWaitTime,
     BadQueueLen,
     DeviceClosed,
+    CallbackSetToNone,
     )
 
-class ValuesMixin(object):
+callback_warning = (
+    'The callback was set to None. This may have been unintentional '
+    'e.g. btn.when_pressed = pressed() instead of btn.when_pressed = pressed'
+)
+
+
+class ValuesMixin:
     """
     Adds a :attr:`values` property to the class which returns an infinite
-    generator of readings from the :attr:`value` property. There is rarely a
-    need to use this mixin directly as all base classes in GPIO Zero include
-    it.
+    generator of readings from the :attr:`~Device.value` property. There is
+    rarely a need to use this mixin directly as all base classes in GPIO Zero
+    include it.
 
     .. note::
 
@@ -41,7 +47,7 @@ class ValuesMixin(object):
     @property
     def values(self):
         """
-        An infinite iterator of values read from `value`.
+        An infinite iterator of values read from :attr:`value`.
         """
         while True:
             try:
@@ -50,12 +56,12 @@ class ValuesMixin(object):
                 break
 
 
-class SourceMixin(object):
+class SourceMixin:
     """
-    Adds a :attr:`source` property to the class which, given an iterable, sets
-    :attr:`value` to each member of that iterable until it is exhausted.  This
-    mixin is generally included in novel output devices to allow their state to
-    be driven from another device.
+    Adds a :attr:`source` property to the class which, given an iterable or a
+    :class:`ValuesMixin` descendent, sets :attr:`~Device.value` to each member
+    of that iterable until it is exhausted. This mixin is generally included in
+    novel output devices to allow their state to be driven from another device.
 
     .. note::
 
@@ -66,17 +72,11 @@ class SourceMixin(object):
         self._source = None
         self._source_thread = None
         self._source_delay = 0.01
-        super(SourceMixin, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def close(self):
-        try:
-            self.source = None
-        except AttributeError:
-            pass
-        try:
-            super(SourceMixin, self).close()
-        except AttributeError:
-            pass
+        self.source = None
+        super().close()
 
     def _copy_values(self, source):
         for v in source:
@@ -111,13 +111,15 @@ class SourceMixin(object):
         if getattr(self, '_source_thread', None):
             self._source_thread.stop()
         self._source_thread = None
+        if isinstance(value, ValuesMixin):
+            value = value.values
         self._source = value
         if value is not None:
-            self._source_thread = GPIOThread(target=self._copy_values, args=(value,))
+            self._source_thread = GPIOThread(self._copy_values, (value,))
             self._source_thread.start()
 
 
-class SharedMixin(object):
+class SharedMixin:
     """
     This mixin marks a class as "shared". In this case, the meta-class
     (GPIOMeta) will use :meth:`_shared_key` to convert the constructor
@@ -127,136 +129,41 @@ class SharedMixin(object):
     used to determine how many times an instance has been "constructed" in this
     way.
 
-    When :meth:`close` is called, an internal reference counter will be
+    When :meth:`~Device.close` is called, an internal reference counter will be
     decremented and the instance will only close when it reaches zero.
     """
     _instances = {}
 
     def __del__(self):
         self._refs = 0
-        super(SharedMixin, self).__del__()
+        super().__del__()
 
     @classmethod
     def _shared_key(cls, *args, **kwargs):
         """
-        Given the constructor arguments, returns an immutable key representing
-        the instance. The default simply assumes all positional arguments are
-        immutable.
+        This is called with the constructor arguments to generate a unique
+        key (which must be storable in a :class:`dict` and, thus, immutable
+        and hashable) representing the instance that can be shared. This must
+        be overridden by descendents.
         """
-        return args
+        raise NotImplementedError
 
 
-class EventsMixin(object):
+class event:
     """
-    Adds edge-detected :meth:`when_activated` and :meth:`when_deactivated`
-    events to a device based on changes to the :attr:`~Device.is_active`
-    property common to all devices. Also adds :meth:`wait_for_active` and
-    :meth:`wait_for_inactive` methods for level-waiting.
+    A descriptor representing a callable event on a class descending from
+    :class:`EventsMixin`.
 
-    .. note::
-
-        Note that this mixin provides no means of actually firing its events;
-        call :meth:`_fire_events` in sub-classes when device state changes to
-        trigger the events. This should also be called once at the end of
-        initialization to set initial states.
+    Instances of this class are very similar to a :class:`property` but also
+    deal with notifying the owning class when events are assigned (or
+    unassigned) and wrapping callbacks implicitly as appropriate.
     """
-    def __init__(self, *args, **kwargs):
-        super(EventsMixin, self).__init__(*args, **kwargs)
-        self._active_event = Event()
-        self._inactive_event = Event()
-        self._when_activated = None
-        self._when_deactivated = None
-        self._last_state = None
-        self._last_changed = time()
+    def __init__(self, doc=None):
+        self.handlers = {}
+        self.__doc__ = doc
 
-    def wait_for_active(self, timeout=None):
-        """
-        Pause the script until the device is activated, or the timeout is
-        reached.
-
-        :param float timeout:
-            Number of seconds to wait before proceeding. If this is ``None``
-            (the default), then wait indefinitely until the device is active.
-        """
-        return self._active_event.wait(timeout)
-
-    def wait_for_inactive(self, timeout=None):
-        """
-        Pause the script until the device is deactivated, or the timeout is
-        reached.
-
-        :param float timeout:
-            Number of seconds to wait before proceeding. If this is ``None``
-            (the default), then wait indefinitely until the device is inactive.
-        """
-        return self._inactive_event.wait(timeout)
-
-    @property
-    def when_activated(self):
-        """
-        The function to run when the device changes state from inactive to
-        active.
-
-        This can be set to a function which accepts no (mandatory) parameters,
-        or a Python function which accepts a single mandatory parameter (with
-        as many optional parameters as you like). If the function accepts a
-        single mandatory parameter, the device that activated will be passed
-        as that parameter.
-
-        Set this property to ``None`` (the default) to disable the event.
-        """
-        return self._when_activated
-
-    @when_activated.setter
-    def when_activated(self, value):
-        self._when_activated = self._wrap_callback(value)
-
-    @property
-    def when_deactivated(self):
-        """
-        The function to run when the device changes state from active to
-        inactive.
-
-        This can be set to a function which accepts no (mandatory) parameters,
-        or a Python function which accepts a single mandatory parameter (with
-        as many optional parameters as you like). If the function accepts a
-        single mandatory parameter, the device that deactivated will be
-        passed as that parameter.
-
-        Set this property to ``None`` (the default) to disable the event.
-        """
-        return self._when_deactivated
-
-    @when_deactivated.setter
-    def when_deactivated(self, value):
-        self._when_deactivated = self._wrap_callback(value)
-
-    @property
-    def active_time(self):
-        """
-        The length of time (in seconds) that the device has been active for.
-        When the device is inactive, this is ``None``.
-        """
-        if self._active_event.is_set():
-            return time() - self._last_changed
-        else:
-            return None
-
-    @property
-    def inactive_time(self):
-        """
-        The length of time (in seconds) that the device has been inactive for.
-        When the device is active, this is ``None``.
-        """
-        if self._inactive_event.is_set():
-            return time() - self._last_changed
-        else:
-            return None
-
-    def _wrap_callback(self, fn):
-        if fn is None:
-            return None
-        elif not callable(fn):
+    def _wrap_callback(self, instance, fn):
+        if not callable(fn):
             raise BadEventHandler('value must be None or a callable')
         # If fn is wrapped with partial (i.e. partial, partialmethod, or wraps
         # has been used to produce it) we need to dig out the "real" function
@@ -285,16 +192,156 @@ class EventsMixin(object):
                 try:
                     # If the above fails, try binding with a single parameter
                     # (ourselves). If this works, wrap the specified callback
-                    inspect.getcallargs(wrapped_fn, *(args + (self,)))
+                    inspect.getcallargs(wrapped_fn, *(args + (instance,)))
                     @wraps(fn)
                     def wrapper():
-                        return fn(self)
+                        return fn(instance)
                     return wrapper
                 except TypeError:
                     raise BadEventHandler(
                         'value must be a callable which accepts up to one '
                         'mandatory parameter')
 
+    def __get__(self, instance, owner=None):
+        if instance is None:
+            return self
+        else:
+            return self.handlers.get(id(instance))
+
+    def __set__(self, instance, value):
+        if value is None:
+            try:
+                del self.handlers[id(instance)]
+            except KeyError:
+                warnings.warn(CallbackSetToNone(callback_warning))
+        else:
+            self.handlers[id(instance)] = self._wrap_callback(instance, value)
+        enabled = any(
+            obj.handlers.get(id(instance))
+            for name in dir(type(instance))
+            for obj in (getattr(type(instance), name),)
+            if isinstance(obj, event)
+        )
+        instance._start_stop_events(enabled)
+
+
+class EventsMixin:
+    """
+    Adds edge-detected :meth:`when_activated` and :meth:`when_deactivated`
+    events to a device based on changes to the :attr:`~Device.is_active`
+    property common to all devices. Also adds :meth:`wait_for_active` and
+    :meth:`wait_for_inactive` methods for level-waiting.
+
+    .. note::
+
+        Note that this mixin provides no means of actually firing its events;
+        call :meth:`_fire_events` in sub-classes when device state changes to
+        trigger the events. This should also be called once at the end of
+        initialization to set initial states.
+    """
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._active_event = Event()
+        self._inactive_event = Event()
+        self._last_active = None
+        self._last_changed = self.pin_factory.ticks()
+
+    def _all_events(self):
+        """
+        Generator function which yields all :class:`event` instances defined
+        against this class.
+        """
+        for name in dir(type(self)):
+            obj = getattr(type(self), name)
+            if isinstance(obj, event):
+                yield obj
+
+    def close(self):
+        for ev in self._all_events():
+            try:
+                del ev.handlers[id(self)]
+            except KeyError:
+                pass
+        super().close()
+
+    def wait_for_active(self, timeout=None):
+        """
+        Pause the script until the device is activated, or the timeout is
+        reached.
+
+        :type timeout: float or None
+        :param timeout:
+            Number of seconds to wait before proceeding. If this is
+            :data:`None` (the default), then wait indefinitely until the device
+            is active.
+        """
+        return self._active_event.wait(timeout)
+
+    def wait_for_inactive(self, timeout=None):
+        """
+        Pause the script until the device is deactivated, or the timeout is
+        reached.
+
+        :type timeout: float or None
+        :param timeout:
+            Number of seconds to wait before proceeding. If this is
+            :data:`None` (the default), then wait indefinitely until the device
+            is inactive.
+        """
+        return self._inactive_event.wait(timeout)
+
+    when_activated = event(
+        """
+        The function to run when the device changes state from inactive to
+        active.
+
+        This can be set to a function which accepts no (mandatory) parameters,
+        or a Python function which accepts a single mandatory parameter (with
+        as many optional parameters as you like). If the function accepts a
+        single mandatory parameter, the device that activated it will be passed
+        as that parameter.
+
+        Set this property to :data:`None` (the default) to disable the event.
+        """)
+
+    when_deactivated = event(
+        """
+        The function to run when the device changes state from active to
+        inactive.
+
+        This can be set to a function which accepts no (mandatory) parameters,
+        or a Python function which accepts a single mandatory parameter (with
+        as many optional parameters as you like). If the function accepts a
+        single mandatory parameter, the device that deactivated it will be
+        passed as that parameter.
+
+        Set this property to :data:`None` (the default) to disable the event.
+        """)
+
+    @property
+    def active_time(self):
+        """
+        The length of time (in seconds) that the device has been active for.
+        When the device is inactive, this is :data:`None`.
+        """
+        if self._active_event.is_set():
+            return self.pin_factory.ticks_diff(self.pin_factory.ticks(),
+                                               self._last_changed)
+        else:
+            return None
+
+    @property
+    def inactive_time(self):
+        """
+        The length of time (in seconds) that the device has been inactive for.
+        When the device is active, this is :data:`None`.
+        """
+        if self._inactive_event.is_set():
+            return self.pin_factory.ticks_diff(self.pin_factory.ticks(),
+                                               self._last_changed)
+        else:
+            return None
+
     def _fire_activated(self):
         # These methods are largely here to be overridden by descendents
         if self.when_activated:
@@ -305,19 +352,34 @@ class EventsMixin(object):
         if self.when_deactivated:
             self.when_deactivated()
 
-    def _fire_events(self):
-        old_state = self._last_state
-        new_state = self._last_state = self.is_active
-        if old_state is None:
+    def _fire_events(self, ticks, new_active):
+        """
+        This method should be called by descendents whenever the
+        :attr:`~Device.is_active` property is likely to have changed (for
+        example, in response to a pin's :attr:`~gpiozero.Pin.state` changing).
+
+        The *ticks* parameter must be set to the time when the change occurred;
+        this can usually be obtained from the pin factory's
+        :meth:`gpiozero.Factory.ticks` method but some pin implementations will
+        implicitly provide the ticks when an event occurs as part of their
+        reporting mechanism.
+
+        The *new_active* parameter must be set to the device's
+        :attr:`~Device.is_active` value at the time indicated by *ticks* (which
+        is not necessarily the value of :attr:`~Device.is_active` right now, if
+        the pin factory provides means of reporting a pin's historical state).
+        """
+        old_active, self._last_active = self._last_active, new_active
+        if old_active is None:
             # Initial "indeterminate" state; set events but don't fire
             # callbacks as there's not necessarily an edge
-            if new_state:
+            if new_active:
                 self._active_event.set()
             else:
                 self._inactive_event.set()
-        elif old_state != new_state:
-            self._last_changed = time()
-            if new_state:
+        elif old_active != new_active:
+            self._last_changed = ticks
+            if new_active:
                 self._inactive_event.clear()
                 self._active_event.set()
                 self._fire_activated()
@@ -326,16 +388,33 @@ class EventsMixin(object):
                 self._inactive_event.set()
                 self._fire_deactivated()
 
+    def _start_stop_events(self, enabled):
+        """
+        This is a stub method that only exists to be overridden by descendents.
+        It is called when :class:`event` properties are assigned (including
+        when set to :data:`None) to permit the owning instance to activate or
+        deactivate monitoring facilities.
+
+        For example, if a descendent requires a background thread to monitor a
+        device, it would be preferable to only run the thread if event handlers
+        are present to respond to it.
+
+        The *enabled* parameter is :data:`False` when all :class:`event`
+        properties on the owning class are :data:`None`, and :data:`True`
+        otherwise.
+        """
+        pass
+
 
 class HoldMixin(EventsMixin):
     """
     Extends :class:`EventsMixin` to add the :attr:`when_held` event and the
     machinery to fire that event repeatedly (when :attr:`hold_repeat` is
-    ``True``) at internals defined by :attr:`hold_time`.
+    :data:`True`) at internals defined by :attr:`hold_time`.
     """
     def __init__(self, *args, **kwargs):
         self._hold_thread = None
-        super(HoldMixin, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._when_held = None
         self._held_from = None
         self._hold_time = 1
@@ -343,28 +422,24 @@ class HoldMixin(EventsMixin):
         self._hold_thread = HoldThread(self)
 
     def close(self):
-        if getattr(self, '_hold_thread', None):
+        if self._hold_thread is not None:
             self._hold_thread.stop()
         self._hold_thread = None
-        try:
-            super(HoldMixin, self).close()
-        except AttributeError:
-            pass
+        super().close()
 
     def _fire_activated(self):
-        super(HoldMixin, self)._fire_activated()
+        super()._fire_activated()
         self._hold_thread.holding.set()
 
     def _fire_deactivated(self):
         self._held_from = None
-        super(HoldMixin, self)._fire_deactivated()
+        super()._fire_deactivated()
 
     def _fire_held(self):
         if self.when_held:
             self.when_held()
 
-    @property
-    def when_held(self):
+    when_held = event(
         """
         The function to run when the device has remained active for
         :attr:`hold_time` seconds.
@@ -375,13 +450,8 @@ class HoldMixin(EventsMixin):
         single mandatory parameter, the device that activated will be passed
         as that parameter.
 
-        Set this property to ``None`` (the default) to disable the event.
-        """
-        return self._when_held
-
-    @when_held.setter
-    def when_held(self, value):
-        self._when_held = self._wrap_callback(value)
+        Set this property to :data:`None` (the default) to disable the event.
+        """)
 
     @property
     def hold_time(self):
@@ -402,7 +472,7 @@ class HoldMixin(EventsMixin):
     @property
     def hold_repeat(self):
         """
-        If ``True``, :attr:`when_held` will be executed repeatedly with
+        If :data:`True`, :attr:`when_held` will be executed repeatedly with
         :attr:`hold_time` seconds between each invocation.
         """
         return self._hold_repeat
@@ -414,7 +484,7 @@ class HoldMixin(EventsMixin):
     @property
     def is_held(self):
         """
-        When ``True``, the device has been active for at least
+        When :data:`True`, the device has been active for at least
         :attr:`hold_time` seconds.
         """
         return self._held_from is not None
@@ -426,10 +496,11 @@ class HoldMixin(EventsMixin):
         This is counted from the first execution of the :attr:`when_held` event
         rather than when the device activated, in contrast to
         :attr:`~EventsMixin.active_time`. If the device is not currently held,
-        this is ``None``.
+        this is :data:`None`.
         """
         if self._held_from is not None:
-            return time() - self._held_from
+            return self.pin_factory.ticks_diff(self.pin_factory.ticks(),
+                                               self._held_from)
         else:
             return None
 
@@ -441,7 +512,7 @@ class HoldThread(GPIOThread):
     device is active.
     """
     def __init__(self, parent):
-        super(HoldThread, self).__init__(
+        super().__init__(
             target=self.held, args=(weakref.proxy(parent),))
         self.holding = Event()
         self.start()
@@ -456,7 +527,7 @@ class HoldThread(GPIOThread):
                             parent._inactive_event.wait(parent.hold_time)
                             ):
                         if parent._held_from is None:
-                            parent._held_from = time()
+                            parent._held_from = parent.pin_factory.ticks()
                         parent._fire_held()
                         if not parent.hold_repeat:
                             break
@@ -475,19 +546,22 @@ class GPIOQueue(GPIOThread):
     """
     def __init__(
             self, parent, queue_len=5, sample_wait=0.0, partial=False,
-            average=median):
+            average=median, ignore=None):
         assert callable(average)
-        super(GPIOQueue, self).__init__(target=self.fill)
         if queue_len < 1:
             raise BadQueueLen('queue_len must be at least one')
         if sample_wait < 0:
             raise BadWaitTime('sample_wait must be 0 or greater')
+        if ignore is None:
+            ignore = set()
+        super().__init__(target=self.fill)
         self.queue = deque(maxlen=queue_len)
         self.partial = bool(partial)
         self.sample_wait = float(sample_wait)
         self.full = Event()
         self.parent = weakref.proxy(parent)
         self.average = average
+        self.ignore = ignore
 
     @property
     def value(self):
@@ -495,19 +569,20 @@ class GPIOQueue(GPIOThread):
             self.full.wait()
         try:
             return self.average(self.queue)
-        except ZeroDivisionError:
+        except (ZeroDivisionError, ValueError):
             # No data == inactive value
             return 0.0
 
     def fill(self):
         try:
             while not self.stopping.wait(self.sample_wait):
-                self.queue.append(self.parent._read())
+                value = self.parent._read()
+                if value not in self.ignore:
+                    self.queue.append(value)
                 if not self.full.is_set() and len(self.queue) >= self.queue.maxlen:
                     self.full.set()
                 if (self.partial or self.full.is_set()) and isinstance(self.parent, EventsMixin):
-                    self.parent._fire_events()
+                    self.parent._fire_events(self.parent.pin_factory.ticks(), self.parent.is_active)
         except ReferenceError:
             # Parent is dead; time to die!
             pass
-
