@@ -2,24 +2,12 @@
 #
 # GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
 #
-# Copyright (c) 2015-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2015-2023 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2020 Fangchen Li <fangchen.li@outlook.com>
 # Copyright (c) 2016-2020 Andrew Scheller <github@loowis.durge.org>
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
-from __future__ import (
-    unicode_literals,
-    absolute_import,
-    print_function,
-    division,
-    )
-try:
-    range = xrange
-except NameError:
-    pass
-nstr = str
-str = type('')
-
 import io
 import os
 import sys
@@ -27,14 +15,10 @@ import mmap
 import errno
 import struct
 import select
-import warnings
 from time import sleep
 from threading import Thread, Event, RLock
-from collections import Counter
-try:
-    from queue import Queue, Empty
-except ImportError:
-    from Queue import Queue, Empty
+from queue import Queue, Empty
+from pathlib import Path
 
 from .local import LocalPiPin, LocalPiFactory
 from ..exc import (
@@ -43,42 +27,44 @@ from ..exc import (
     PinInvalidFunction,
     PinFixedPull,
     PinSetInput,
-    )
+)
 
 
-def dt_resolve_alias(alias, root='/proc/device-tree'):
+def dt_resolve_alias(alias, root=Path('/proc/device-tree')):
     """
-    Returns the full path of a device-tree alias. For example:
+    Returns the full :class:`~pathlib.Path` of a device-tree alias. For
+    example:
 
         >>> dt_resolve_alias('gpio')
         '/proc/device-tree/soc/gpio@7e200000'
         >>> dt_resolve_alias('ethernet0', root='/proc/device-tree')
         '/proc/device-tree/scb/ethernet@7d580000'
     """
-    # XXX Change this return a pathlib.Path when we drop 2.x
-    filename = os.path.join(root, 'aliases', alias)
-    with io.open(filename, 'rb') as f:
+    if not isinstance(root, Path):
+        root = Path(root)
+    filename = root / 'aliases' / alias
+    with filename.open('rb') as f:
         node, tail = f.read().split(b'\0', 1)
         fs_encoding = sys.getfilesystemencoding()
-        return os.path.join(root, node.decode(fs_encoding).lstrip('/'))
+        return root / node.decode(fs_encoding).lstrip('/')
 
-def dt_peripheral_reg(node, root='/proc/device-tree'):
+def dt_peripheral_reg(node, root=Path('/proc/device-tree')):
     """
     Returns the :class:`range` covering the registers of the specified *node*
     of the device-tree, mapped to the CPU's address space. For example:
 
         >>> reg = dt_peripheral_reg(dt_resolve_alias('gpio'))
-        >>> '%#x..%#x' % (reg.start, reg.stop)
+        >>> f'{reg.start:#x}..{reg.stop:#x}'
         '0xfe200000..0xfe2000b4'
         >>> hex(dt_peripheral_reg(dt_resolve_alias('ethernet0')).start)
         '0xfd580000'
     """
     # Returns a tuple of (address-cells, size-cells) for *node*
     def _cells(node):
-        with io.open(os.path.join(node, '#address-cells'), 'rb') as f:
-            address_cells = struct.unpack(nstr('>L'), f.read())[0]
-        with io.open(os.path.join(node, '#size-cells'), 'rb') as f:
-            size_cells = struct.unpack(nstr('>L'), f.read())[0]
+        with (node / '#address-cells').open('rb') as f:
+            address_cells = struct.unpack('>L', f.read())[0]
+        with (node / '#size-cells').open('rb') as f:
+            size_cells = struct.unpack('>L', f.read())[0]
         return (address_cells, size_cells)
 
     # Returns a generator function which, given a file-like object *source*
@@ -86,8 +72,7 @@ def dt_peripheral_reg(node, root='/proc/device-tree'):
     # contains one integer for each specified *length*, which is the number of
     # 32-bit device-tree cells that make up that value.
     def _reader(*lengths):
-        structs = [struct.Struct(nstr('>{cells}L'.format(cells=cells)))
-                   for cells in lengths]
+        structs = [struct.Struct(f'>{cells}L') for cells in lengths]
         offsets = [sum(s.size for s in structs[:i])
                    for i in range(len(structs))]
         buf_len = sum(s.size for s in structs)
@@ -98,8 +83,7 @@ def dt_peripheral_reg(node, root='/proc/device-tree'):
                 if not buf:
                     break
                 elif len(buf) < buf_len:
-                    raise IOError('failed to read {buf_len} bytes'.format(
-                        buf_len=buf_len))
+                    raise IOError(f'failed to read {buf_len} bytes')
                 row = ()
                 for offset, s in zip(offsets, structs):
                     cells = s.unpack_from(buf, offset)
@@ -113,37 +97,34 @@ def dt_peripheral_reg(node, root='/proc/device-tree'):
     # Returns a list of (child-range, parent-range) tuples for *node*
     def _ranges(node):
         child_cells, size_cells = _cells(node)
-        parent = os.path.dirname(node)
-        parent_cells, _ = _cells(parent)
+        parent_cells, _ = _cells(node.parent)
         ranges_reader = _reader(child_cells, parent_cells, size_cells)
-        with io.open(os.path.join(node, 'ranges'), 'rb') as f:
+        with (node / 'ranges').open('rb') as f:
             return [
                 (range(child_base, child_base + size),
                  range(parent_base, parent_base + size))
                 for child_base, parent_base, size in ranges_reader(f)
             ]
 
-    # XXX Replace all this gubbins with pathlib.Path stuff once we drop 2.x
-    node = os.path.join(root, node)
-    parent = os.path.dirname(node)
-    child_cells, size_cells = _cells(parent)
+    if not isinstance(root, Path):
+        root = Path(root)
+    node = root / node
+    child_cells, size_cells = _cells(node.parent)
     reg_reader = _reader(child_cells, size_cells)
-    with io.open(os.path.join(node, 'reg'), 'rb') as f:
+    with (node / 'reg').open('rb') as f:
         base, size = list(reg_reader(f))[0]
-    while parent != root:
+    while node.parent != root:
         # Iterate up the hierarchy, resolving the base address as we go
-        if os.path.exists(os.path.join(parent, 'ranges')):
-            for child_range, parent_range in _ranges(parent):
+        if (node.parent / 'ranges').exists():
+            for child_range, parent_range in _ranges(node.parent):
                 if base in child_range:
-                    # XXX Can't use .start here as python2's crappy xrange
-                    # lacks it; change this when we drop 2.x!
-                    base += parent_range[0] - child_range[0]
+                    base += parent_range.start - child_range.start
                     break
-        parent = os.path.dirname(parent)
+        node = node.parent
     return range(base, base + size)
 
 
-class GPIOMemory(object):
+class GPIOMemory:
     GPIO_BASE_OFFSET = 0x200000
     PERI_BASE_OFFSET = {
         'BCM2835': 0x20000000,
@@ -191,7 +172,7 @@ class GPIOMemory(object):
         try:
             self.reg_fmt = {
                 struct.calcsize(fmt): fmt
-                for fmt in (nstr('@I'), nstr('@L'))
+                for fmt in ('@I', '@L')
             }[4]
         except KeyError:
             raise RuntimeError('unable to find native unsigned 32-bit type')
@@ -202,8 +183,7 @@ class GPIOMemory(object):
 
     def gpio_base(self, soc):
         try:
-            # XXX Replace this with .start when 2.x is dropped
-            return dt_peripheral_reg(dt_resolve_alias('gpio'))[0]
+            return dt_peripheral_reg(dt_resolve_alias('gpio')).start
         except IOError:
             try:
                 return self.PERI_BASE_OFFSET[soc] + self.GPIO_BASE_OFFSET
@@ -218,7 +198,7 @@ class GPIOMemory(object):
         struct.pack_into(self.reg_fmt, self.mem, index * 4, value)
 
 
-class GPIOFS(object):
+class GPIOFS:
     GPIO_PATH = '/sys/class/gpio'
 
     def __init__(self, factory, queue):
@@ -240,13 +220,13 @@ class GPIOFS(object):
         return os.path.join(self.GPIO_PATH, name)
 
     def path_value(self, pin):
-        return self.path('gpio%d/value' % pin)
+        return self.path(f'gpio{pin:d}/value')
 
     def path_dir(self, pin):
-        return self.path('gpio%d/direction' % pin)
+        return self.path(f'gpio{pin:d}/direction')
 
     def path_edge(self, pin):
-        return self.path('gpio%d/edge' % pin)
+        return self.path(f'gpio{pin:d}/edge')
 
     def exported(self, pin):
         return pin in self._exports
@@ -289,7 +269,7 @@ class GPIOFS(object):
                         else:
                             raise
                 if result is None:
-                    raise RuntimeError('failed to export pin %d' % pin)
+                    raise RuntimeError(f'failed to export pin {pin:d}')
             return result
 
     def unexport(self, pin):
@@ -322,7 +302,7 @@ class GPIOFS(object):
 
 class NativeWatchThread(Thread):
     def __init__(self, factory, queue):
-        super(NativeWatchThread, self).__init__(
+        super().__init__(
             target=self._run, args=(factory, queue))
         self.daemon = True
         self._stop_evt = Event()
@@ -360,7 +340,7 @@ class NativeWatchThread(Thread):
 
 class NativeDispatchThread(Thread):
     def __init__(self, factory, queue):
-        super(NativeDispatchThread, self).__init__(
+        super().__init__(
             target=self._run, args=(factory, queue))
         self.daemon = True
         self._stop_evt = Event()
@@ -410,12 +390,12 @@ class NativeFactory(LocalPiFactory):
         led = LED(12, pin_factory=factory)
     """
     def __init__(self):
-        super(NativeFactory, self).__init__()
+        super().__init__()
         queue = Queue()
-        self.mem = GPIOMemory(self.pi_info.soc)
+        self.mem = GPIOMemory(self.board_info.soc)
         self.fs = GPIOFS(self, queue)
         self.dispatch = NativeDispatchThread(self, queue)
-        if self.pi_info.soc == 'BCM2711':
+        if self.board_info.soc == 'BCM2711':
             self.pin_class = Native2711Pin
         else:
             self.pin_class = Native2835Pin
@@ -424,7 +404,7 @@ class NativeFactory(LocalPiFactory):
         if self.dispatch is not None:
             self.dispatch.close()
             self.dispatch = None
-        super(NativeFactory, self).close()
+        super().close()
         if self.fs is not None:
             self.fs.close()
             self.fs = None
@@ -451,15 +431,15 @@ class NativePin(LocalPiPin):
 
     GPIO_FUNCTION_NAMES = {v: k for (k, v) in GPIO_FUNCTIONS.items()}
 
-    def __init__(self, factory, number):
-        super(NativePin, self).__init__(factory, number)
-        self._reg_init(factory, number)
+    def __init__(self, factory, info):
+        super().__init__(factory, info)
+        self._reg_init(factory, self._number)
         self._last_call = None
         self._when_changed = None
         self._change_thread = None
         self._change_event = Event()
         self.function = 'input'
-        self.pull = 'up' if self.factory.pi_info.pulled_up(repr(self)) else 'floating'
+        self.pull = info.pull or 'floating'
         self.bounce = None
         self.edges = 'none'
 
@@ -484,7 +464,7 @@ class NativePin(LocalPiPin):
         self.frequency = None
         self.when_changed = None
         self.function = 'input'
-        self.pull = 'up' if self.factory.pi_info.pulled_up(repr(self)) else 'floating'
+        self.pull = self.info.pull or 'floating'
 
     def _get_function(self):
         return self.GPIO_FUNCTION_NAMES[(self.factory.mem[self._func_offset] >> self._func_shift) & 7]
@@ -493,19 +473,20 @@ class NativePin(LocalPiPin):
         try:
             value = self.GPIO_FUNCTIONS[value]
         except KeyError:
-            raise PinInvalidFunction('invalid function "%s" for pin %r' % (value, self))
+            raise PinInvalidFunction(
+                f'invalid function "{value}" for pin {self!r}')
         self.factory.mem[self._func_offset] = (
             self.factory.mem[self._func_offset]
             & ~(7 << self._func_shift)
             | (value << self._func_shift)
-            )
+        )
 
     def _get_state(self):
         return bool(self.factory.mem[self._level_offset] & (1 << self._level_shift))
 
     def _set_state(self, value):
         if self.function == 'input':
-            raise PinSetInput('cannot set state of pin %r' % self)
+            raise PinSetInput(f'cannot set state of pin {self!r}')
         if value:
             self.factory.mem[self._set_offset] = 1 << self._set_shift
         else:
@@ -525,7 +506,7 @@ class NativePin(LocalPiPin):
 
     def _get_edges(self):
         try:
-            with io.open(self.factory.fs.path_edge(self.number), 'r') as f:
+            with io.open(self.factory.fs.path_edge(self._number), 'r') as f:
                 return f.read().strip()
         except IOError as e:
             if e.errno == errno.ENOENT:
@@ -535,24 +516,25 @@ class NativePin(LocalPiPin):
 
     def _set_edges(self, value):
         if value != 'none':
-            self.factory.fs.export(self.number)
+            self.factory.fs.export(self._number)
         try:
-            with io.open(self.factory.fs.path_edge(self.number), 'w') as f:
+            with io.open(self.factory.fs.path_edge(self._number), 'w') as f:
                 f.write(value)
         except IOError as e:
             if e.errno == errno.ENOENT and value == 'none':
                 pass
             elif e.errno == errno.EINVAL:
-                raise PinInvalidEdges('invalid edge specification "%s" for pin %r' % self)
+                raise PinInvalidEdges(
+                    f'invalid edge specification "{value}" for pin {self!r}')
             else:
                 raise
 
     def _enable_event_detect(self):
-        self.factory.fs.watch(self.number)
+        self.factory.fs.watch(self._number)
         self._last_call = None
 
     def _disable_event_detect(self):
-        self.factory.fs.unwatch(self.number)
+        self.factory.fs.unwatch(self._number)
 
 
 class Native2835Pin(NativePin):
@@ -569,7 +551,7 @@ class Native2835Pin(NativePin):
     GPIO_PULL_UP_NAMES = {v: k for (k, v) in GPIO_PULL_UPS.items()}
 
     def _reg_init(self, factory, number):
-        super(Native2835Pin, self)._reg_init(factory, number)
+        super()._reg_init(factory, number)
         self._pull_offset = self.factory.mem.GPPUDCLK_OFFSET + (number // 32)
         self._pull_shift = number % 32
         self._pull = 'floating'
@@ -579,13 +561,15 @@ class Native2835Pin(NativePin):
 
     def _set_pull(self, value):
         if self.function != 'input':
-            raise PinFixedPull('cannot set pull on non-input pin %r' % self)
-        if value != 'up' and self.factory.pi_info.pulled_up(repr(self)):
-            raise PinFixedPull('%r has a physical pull-up resistor' % self)
+            raise PinFixedPull(f'cannot set pull on non-input pin {self!r}')
+        if self.info.pull not in (value, ''):
+            raise PinFixedPull(
+                f'{self!r} has a physical pull-{self.info.pull} resistor')
         try:
             value = self.GPIO_PULL_UPS[value]
         except KeyError:
-            raise PinInvalidPull('invalid pull direction "%s" for pin %r' % (value, self))
+            raise PinInvalidPull(
+                f'invalid pull direction "{value}" for pin {self!r}')
         self.factory.mem[self.factory.mem.GPPUD_OFFSET] = value
         sleep(0.000000214)
         self.factory.mem[self._pull_offset] = 1 << self._pull_shift
@@ -609,7 +593,7 @@ class Native2711Pin(NativePin):
     GPIO_PULL_UP_NAMES = {v: k for (k, v) in GPIO_PULL_UPS.items()}
 
     def _reg_init(self, factory, number):
-        super(Native2711Pin, self)._reg_init(factory, number)
+        super()._reg_init(factory, number)
         self._pull_offset = self.factory.mem.GPPUPPDN_OFFSET + (number // 16)
         self._pull_shift = (number % 16) * 2
 
@@ -619,15 +603,17 @@ class Native2711Pin(NativePin):
 
     def _set_pull(self, value):
         if self.function != 'input':
-            raise PinFixedPull('cannot set pull on non-input pin %r' % self)
-        if value != 'up' and self.factory.pi_info.pulled_up(repr(self)):
-            raise PinFixedPull('%r has a physical pull-up resistor' % self)
+            raise PinFixedPull(f'cannot set pull on non-input pin {self!r}')
+        if self.info.pull not in (value, ''):
+            raise PinFixedPull(
+                f'{self!r} has a physical pull-{self.info.pull} resistor')
         try:
             value = self.GPIO_PULL_UPS[value]
         except KeyError:
-            raise PinInvalidPull('invalid pull direction "%s" for pin %r' % (value, self))
+            raise PinInvalidPull(
+                f'invalid pull direction "{value}" for pin {self!r}')
         self.factory.mem[self._pull_offset] = (
             self.factory.mem[self._pull_offset]
             & ~(3 << self._pull_shift)
             | (value << self._pull_shift)
-            )
+        )
