@@ -252,11 +252,14 @@ class PiBoardInfo(BoardInfo):
                 'A':      {'P1': data.REV2_P1, 'P5': data.REV2_P5, 'P6': data.REV2_P6, 'P2': data.PI1_P2, 'P3': data.PI1_P3},
                 'B':      {'P1': data.REV1_P1, 'P2': data.PI1_P2, 'P3': data.PI1_P3} if pcb_revision == '1.0' else
                           {'P1': data.REV2_P1, 'P5': data.REV2_P5, 'P6': data.REV2_P6, 'P2': data.PI1_P2, 'P3': data.PI1_P3},
+                'B+':     {'J8': data.PLUS_J8, 'RUN': data.ZERO_RUN},
                 'CM':     {'SODIMM': data.CM_SODIMM},
                 'CM3':    {'SODIMM': data.CM3_SODIMM},
                 'CM3+':   {'SODIMM': data.CM3_SODIMM},
                 'Zero':   {'J8': data.PLUS_J8, 'RUN': data.ZERO_RUN, 'TV': data.ZERO_TV},
                 'Zero W': {'J8': data.PLUS_J8, 'RUN': data.ZERO_RUN, 'TV': data.ZERO_TV},
+                '2B':     {'J8': data.PLUS_J8, 'RUN': data.ZERO_RUN},
+                '3B':     {'J8': data.PLUS_J8, 'RUN': data.ZERO_RUN},
                 '3A+':    {'J8': data.PLUS_J8, 'RUN': data.PLUS_RUN},
                 '3B+':    {'J8': data.PLUS_J8, 'RUN': data.PLUS_RUN, 'POE': data.PLUS_POE},
                 '4B':     {'J8': data.PI4_J8, 'J2': data.PI4_J2, 'J14': data.PI4_J14},
