@@ -2,33 +2,23 @@
 #
 # GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
 #
-# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2016-2023 Dave Jones <dave@waveform.org.uk>
 # Copyright (c) 2020 Jack Wearden <jack@jackwearden.co.uk>
 # Copyright (c) 2017-2020 Ben Nuttall <ben@bennuttall.com>
 # Copyright (c) 2016-2019 Andrew Scheller <github@loowis.durge.org>
 # Copyright (c) 2018 SteveAmor <steveamor@users.noreply.github.com>
 # Copyright (c) 2018 Claire Pollard <claire.r.pollard@gmail.com>
 # Copyright (c) 2016 Martin OHanlon <martin@ohanlonweb.com>
-# Copyright (c) 2016 Ian Harcombe <ian.harcombe@gmail.com>
 # Copyright (c) 2016 Andrew Scheller <lurch@durge.org>
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
-from __future__ import (
-    unicode_literals,
-    absolute_import,
-    print_function,
-    division,
-    )
-str = type('')
-
-
 import sys
 import pytest
+import warnings
 from time import sleep
 from threading import Event
-
-import mock
+from unittest import mock
 
 from gpiozero import *
 from gpiozero.fonts import *
@@ -908,23 +898,23 @@ def test_traffic_phat(mock_factory):
         assert repr(board).startswith('<gpiozero.TrafficpHat object')
         assert ([led.pin for led in board]) == pins
 
-def test_robot_bad_init(mock_factory):
-    with pytest.raises(GPIOPinMissing):
+def test_robot_bad_init(mock_factory, pwm):
+    pins = [mock_factory.pin(n) for n in (2, 3)]
+    with pytest.raises(TypeError):
         Robot()
     with pytest.raises(GPIOPinMissing):
-        Robot(2)
+        Robot(Motor(2, 3), None)
     with pytest.raises(GPIOPinMissing):
-        Robot(2, 3)
-    with pytest.raises(GPIOPinMissing):
-        Robot(2, 3, 4)
-    with pytest.raises(GPIOPinMissing):
-        Robot(2, 3, 4, 5)
-    with pytest.raises(GPIOPinMissing):
-        Robot(2, 3, 4, 5, 6, 7)
-    with pytest.raises(GPIOPinMissing):
-        Robot((2, 3))
-    with pytest.raises(GPIOPinMissing):
-        Robot((2, 3), 4)
+        Robot(None, Motor(2, 3))
+
+def test_robot_deprecation(mock_factory, pwm):
+    pins = [mock_factory.pin(n) for n in (2, 3, 4, 5)]
+    with warnings.catch_warnings(record=True) as w:
+        warnings.resetwarnings()
+        with Robot((2, 3), (4, 5)) as robot:
+            assert len(w) == 2
+            assert w[0].category == DeprecationWarning
+            assert w[1].category == DeprecationWarning
 
 def test_robot(mock_factory, pwm):
     pins = [mock_factory.pin(n) for n in (2, 3, 4, 5)]
@@ -935,7 +925,7 @@ def test_robot(mock_factory, pwm):
         assert robot.value == (
             pins[0].state - pins[1].state, pins[2].state - pins[3].state
         ) == expected_value
-    with Robot((2, 3), (4, 5)) as robot:
+    with Robot(Motor(2, 3), Motor(4, 5)) as robot:
         assert repr(robot).startswith('<gpiozero.Robot object')
         assert (
             [device.pin for device in robot.left_motor] +
@@ -1104,7 +1094,7 @@ def test_robots(mock_factory, pwm):
 
 def test_robot_nopwm(mock_factory):
     pins = [mock_factory.pin(n) for n in (2, 3, 4, 5)]
-    with Robot((2, 3), (4, 5), pwm=False) as robot:
+    with Robot(Motor(2, 3, pwm=False), Motor(4, 5, pwm=False)) as robot:
         left_motor, right_motor = robot.all
         assert isinstance(left_motor, Motor)
         assert left_motor.forward_device.pin is pins[0]
@@ -1156,7 +1146,7 @@ def test_robots_nopwm(mock_factory):
 
 def test_enable_pin_motor_robot(mock_factory, pwm):
     pins = [mock_factory.pin(n) for n in (2, 3, 4, 5, 6, 7)]
-    with Robot((2, 3, 4), (5, 6, 7)) as robot:
+    with Robot(Motor(2, 3, enable=4), Motor(5, 6, enable=7)) as robot:
         left_motor, right_motor = robot.all
         assert isinstance(left_motor, Motor)
         assert left_motor.forward_device.pin is pins[0]
@@ -1175,7 +1165,8 @@ def test_enable_pin_motor_robot(mock_factory, pwm):
 
 def test_enable_pin_motor_robot_nopwm(mock_factory):
     pins = [mock_factory.pin(n) for n in (2, 3, 4, 5, 6, 7)]
-    with Robot((2, 3, 4), (5, 6, 7), pwm=False) as robot:
+    with Robot(Motor(2, 3, enable=4, pwm=False),
+               Motor(5, 6, enable=7, pwm=False)) as robot:
         left_motor, right_motor = robot.all
         assert isinstance(left_motor, Motor)
         assert left_motor.forward_device.pin is pins[0]
@@ -1192,60 +1183,6 @@ def test_enable_pin_motor_robot_nopwm(mock_factory):
         assert right_motor.enable_device.pin is pins[5]
         assert isinstance(right_motor.enable_device, DigitalOutputDevice)
 
-def test_phaseenable_robot_bad_init(mock_factory):
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableRobot()
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableRobot(2)
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableRobot(2, 3)
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableRobot(2, 3, 4)
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableRobot(2, 3, 4, 5)
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableRobot(2, 3, 4, 5, 6, 7)
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableRobot((2, 3))
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableRobot((2, 3), 4)
-
-def test_phaseenable_robot(mock_factory, pwm):
-    pins = [mock_factory.pin(n) for n in (5, 12, 6, 13)]
-    with PhaseEnableRobot((5, 12), (6, 13)) as robot:
-        assert repr(robot).startswith('<gpiozero.PhaseEnableRobot object')
-        assert (
-            [device.pin for device in robot.left_motor] +
-            [device.pin for device in robot.right_motor]) == pins
-        assert robot.value == (0, 0)
-        robot.forward()
-        assert [pin.state for pin in pins] == [0, 1, 0, 1]
-        assert robot.value == (1, 1)
-        robot.backward()
-        assert [pin.state for pin in pins] == [1, 1, 1, 1]
-        assert robot.value == (-1, -1)
-        robot.forward(0.5)
-        assert [pin.state for pin in pins] == [0, 0.5, 0, 0.5]
-        assert robot.value == (0.5, 0.5)
-        robot.left()
-        assert [pin.state for pin in pins] == [1, 1, 0, 1]
-        assert robot.value == (-1, 1)
-        robot.right()
-        assert [pin.state for pin in pins] == [0, 1, 1, 1]
-        assert robot.value == (1, -1)
-        robot.reverse()
-        assert [pin.state for pin in pins] == [1, 1, 0, 1]
-        assert robot.value == (-1, 1)
-        robot.stop()
-        assert [pin.state for pin in pins][1::2] == [0, 0]
-        assert robot.value == (0, 0)
-        robot.value = (-1, -1)
-        assert robot.value == (-1, -1)
-        robot.value = (0.5, 1)
-        assert robot.value == (0.5, 1)
-        robot.value = (0, -0.5)
-        assert robot.value == (0, -0.5)
-
 def test_energenie_bad_init(mock_factory):
     with pytest.raises(ValueError):
         Energenie()
@@ -1610,9 +1547,9 @@ def test_pibrella_pins(mock_factory, pwm):
     with Pibrella() as pb:
         with LED(pb.outputs.e) as led:
             assert isinstance(led, LED)
-            assert led.pin.number == 22
+            assert led.pin.info.name == 'GPIO22'
         with Button(pb.inputs.a) as btn:
-            assert btn.pin.number == 9
+            assert btn.pin.info.name == 'GPIO9'
 
 def test_led_char_display_init(mock_factory):
     pins = [mock_factory.pin(i) for i in range(28)]
@@ -1670,8 +1607,7 @@ def test_led_char_display_font(mock_factory):
         }
         char.font = font
         assert char.value == 0
-        assert repr(char.font) == """LEDCharFont({
-    0: (1, 1, 1, 1, 1, 1, 0),
+        assert repr(char.font) == """LEDCharFont({   0: (1, 1, 1, 1, 1, 1, 0),
     1: (0, 1, 1, 0, 0, 0, 0),
     2: (1, 1, 0, 1, 1, 0, 1),
     3: (1, 1, 1, 1, 0, 0, 1),
@@ -1680,8 +1616,7 @@ def test_led_char_display_font(mock_factory):
     6: (1, 0, 1, 1, 1, 1, 1),
     7: (1, 1, 1, 0, 0, 0, 0),
     8: (1, 1, 1, 1, 1, 1, 1),
-    9: (1, 1, 1, 1, 0, 1, 1),
-})"""
+    9: (1, 1, 1, 1, 0, 1, 1)})"""
 
 def test_led_multichar_display_init(mock_factory):
     pins = [mock_factory.pin(i) for i in range(10, 22)]
