@@ -2,30 +2,20 @@
 #
 # GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
 #
-# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2016-2023 Dave Jones <dave@waveform.org.uk>
 # Copyright (c) 2019 Ben Nuttall <ben@bennuttall.com>
 # Copyright (c) 2016-2019 Andrew Scheller <github@loowis.durge.org>
 # Copyright (c) 2018 Philippe Muller <philippe.muller@gmail.com>
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
-from __future__ import (
-    unicode_literals,
-    absolute_import,
-    print_function,
-    division,
-    )
-str = type('')
-
-
 import sys
 import pytest
 import warnings
 from time import sleep
 from threading import Event
 from functools import partial
-
-import mock
+from unittest import mock
 
 from conftest import ThreadedTest
 from gpiozero.pins.mock import MockChargingPin, MockTriggerPin
@@ -94,12 +84,12 @@ def test_input_is_active_high_externally_pulled_down(mock_factory):
 def test_input_invalid_pull_up(mock_factory):
     with pytest.raises(PinInvalidState) as exc:
         InputDevice(4, pull_up=None)
-    assert str(exc.value) == 'Pin 4 is defined as floating, but "active_state" is not defined'
+    assert str(exc.value) == 'Pin GPIO4 is defined as floating, but "active_state" is not defined'
 
 def test_input_invalid_active_state(mock_factory):
     with pytest.raises(PinInvalidState) as exc:
         InputDevice(4, active_state=True)
-    assert str(exc.value) == 'Pin 4 is not floating, but "active_state" is not None'
+    assert str(exc.value) == 'Pin GPIO4 is not floating, but "active_state" is not None'
 
 def test_input_event_activated(mock_factory):
     event = Event()
@@ -122,8 +112,6 @@ def test_input_event_deactivated(mock_factory):
         pin.drive_low()
         assert event.is_set()
 
-@pytest.mark.xfail(sys.version_info < (3, 0),
-                   reason='warnings fail to reset properly on py2.7')
 def test_input_activated_callback_warning(mock_factory):
     def foo(): pass
 
@@ -281,6 +269,7 @@ def test_input_light_sensor(mock_factory):
 
 @pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                     reason='timing is too random on pypy')
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.PWMSoftwareFallback')
 def test_input_distance_sensor(mock_factory):
     echo_pin = mock_factory.pin(4)
     trig_pin = mock_factory.pin(5, pin_class=MockTriggerPin,
@@ -477,8 +466,6 @@ def test_input_rotary_encoder_when(mock_factory):
         assert not rotated_cw.wait(0)
         assert rotated_ccw.wait(0)
 
-@pytest.mark.xfail(sys.version_info < (3, 0),
-                   reason="Threaded tests fail on 2.7; no idea why...")
 def test_input_rotary_encoder_wait(mock_factory):
     a_pin = mock_factory.pin(20)
     b_pin = mock_factory.pin(21)
