@@ -2,27 +2,16 @@
 #
 # GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
 #
-# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2016-2023 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2020 Fangchen Li <fangchen.li@outlook.com>
 # Copyright (c) 2018-2019 Ben Nuttall <ben@bennuttall.com>
 # Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
-from __future__ import (
-    unicode_literals,
-    absolute_import,
-    print_function,
-    division,
-    )
-str = type('')
-
-
 import sys
 from time import sleep, time
-try:
-    from math import isclose
-except ImportError:
-    from gpiozero.compat import isclose
+from math import isclose
 
 import pytest
 from colorzero import Color, Red, Green, Blue
@@ -967,18 +956,6 @@ def test_rgbled_close_nonpwm(mock_factory):
         led.close()
         assert led.closed
 
-def test_motor_bad_init(mock_factory):
-    with pytest.raises(GPIOPinMissing):
-        Motor()
-    with pytest.raises(GPIOPinMissing):
-        Motor(2)
-    with pytest.raises(GPIOPinMissing):
-        Motor(forward=2)
-    with pytest.raises(GPIOPinMissing):
-        Motor(backward=2)
-    with pytest.raises(TypeError):
-        Motor(a=2, b=3)
-
 def test_motor_pins(mock_factory, pwm):
     f = mock_factory.pin(1)
     b = mock_factory.pin(2)
@@ -1122,16 +1099,6 @@ def test_motor_reverse_nonpwm(mock_factory):
         assert motor.value == -1
         assert b.state == 1 and f.state == 0
 
-def test_motor_enable_pin_bad_init(mock_factory, pwm):
-    with pytest.raises(GPIOPinMissing):
-        Motor(enable=1)
-    with pytest.raises(GPIOPinMissing):
-        Motor(forward=1, enable=2)
-    with pytest.raises(GPIOPinMissing):
-        Motor(backward=1, enable=2)
-    with pytest.raises(GPIOPinMissing):
-        Motor(backward=1, enable=2, pwm=True)
-
 def test_motor_enable_pin_init(mock_factory, pwm):
     f = mock_factory.pin(1)
     b = mock_factory.pin(2)
@@ -1144,7 +1111,7 @@ def test_motor_enable_pin_init(mock_factory, pwm):
         assert motor.enable_device.pin is e
         assert isinstance(motor.enable_device, DigitalOutputDevice)
         assert e.state
-    with Motor(1, 2, 3) as motor:
+    with Motor(1, 2, enable=3) as motor:
         assert motor.forward_device.pin is f
         assert isinstance(motor.forward_device, PWMOutputDevice)
         assert motor.backward_device.pin is b
@@ -1174,18 +1141,6 @@ def test_motor_enable_pin(mock_factory, pwm):
         motor.stop()
         assert motor.value == 0
 
-def test_phaseenable_motor_bad_init(mock_factory):
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableMotor()
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableMotor(2)
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableMotor(phase=2)
-    with pytest.raises(GPIOPinMissing):
-        PhaseEnableMotor(enable=2)
-    with pytest.raises(TypeError):
-        PhaseEnableMotor(a=2, b=3)
-
 def test_phaseenable_motor_pins(mock_factory, pwm):
     p = mock_factory.pin(1)
     e = mock_factory.pin(2)
@@ -1321,6 +1276,7 @@ def test_phaseenable_motor_reverse_nonpwm(mock_factory):
         assert motor.value == -1
         assert p.state == 1 and e.state == 1
 
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.PWMSoftwareFallback')
 def test_servo_pins(mock_factory, pwm):
     p = mock_factory.pin(1)
     with Servo(1) as servo:
@@ -1328,6 +1284,7 @@ def test_servo_pins(mock_factory, pwm):
         assert servo.pwm_device.pin is p
         assert isinstance(servo.pwm_device, PWMOutputDevice)
 
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.PWMSoftwareFallback')
 def test_servo_bad_value(mock_factory, pwm):
     p = mock_factory.pin(1)
     with pytest.raises(ValueError):
@@ -1342,6 +1299,7 @@ def test_servo_pins_nonpwm(mock_factory):
     with pytest.raises(PinPWMUnsupported):
         Servo(1)
 
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.PWMSoftwareFallback')
 def test_servo_close(mock_factory, pwm):
     p = mock_factory.pin(2)
     with Servo(2) as servo:
@@ -1351,6 +1309,7 @@ def test_servo_close(mock_factory, pwm):
         servo.close()
         assert servo.closed
 
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.PWMSoftwareFallback')
 def test_servo_pulse_width(mock_factory, pwm):
     p = mock_factory.pin(2)
     with Servo(2, min_pulse_width=5/10000, max_pulse_width=25/10000) as servo:
@@ -1367,6 +1326,7 @@ def test_servo_pulse_width(mock_factory, pwm):
         servo.value = None
         assert servo.pulse_width is None
 
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.PWMSoftwareFallback')
 def test_servo_initial_values(mock_factory, pwm):
     p = mock_factory.pin(2)
     with Servo(2) as servo:
@@ -1387,6 +1347,7 @@ def test_servo_initial_values(mock_factory, pwm):
         assert not servo.is_active
         assert servo.value is None
 
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.PWMSoftwareFallback')
 def test_servo_values(mock_factory, pwm):
     p = mock_factory.pin(1)
     with Servo(1) as servo:
@@ -1421,12 +1382,14 @@ def test_angular_servo_bad_value(mock_factory, pwm):
     with pytest.raises(ValueError):
         AngularServo(1, min_angle=95)
 
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.PWMSoftwareFallback')
 def test_angular_servo_range(mock_factory, pwm):
     with AngularServo(1, initial_angle=15, min_angle=0, max_angle=90) as servo:
         assert repr(servo).startswith('<gpiozero.AngularServo object')
         assert servo.min_angle == 0
         assert servo.max_angle == 90
 
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.PWMSoftwareFallback')
 def test_angular_servo_initial_angles(mock_factory, pwm):
     with AngularServo(1) as servo:
         assert servo.angle == 0
@@ -1442,6 +1405,7 @@ def test_angular_servo_initial_angles(mock_factory, pwm):
     with AngularServo(1, initial_angle=None) as servo:
         assert servo.angle is None
 
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.PWMSoftwareFallback')
 def test_angular_servo_angles(mock_factory, pwm):
     with AngularServo(1) as servo:
         servo.angle = 0
@@ -1530,6 +1494,7 @@ def test_tonalbuzzer_init(mock_factory, pwm):
     with TonalBuzzer(2, octaves=2, initial_value=1) as tb:
         assert isclose(tb.pwm_device.frequency, 1760)
 
+@pytest.mark.filterwarnings('ignore::gpiozero.exc.AmbiguousTone')
 def test_tonalbuzzer_play(mock_factory, pwm):
     with TonalBuzzer(2) as tb:
         tb.play(60)
